package pejb.rmi.chat.client;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.rmi.*;
import java.rmi.server.*;
import pejb.rmi.base.RmiUtil;
import pejb.rmi.chat.server.Server;

public class ChatClient extends JApplet implements Receiver, ActionListener {
	private String login_name="", host="";
	private boolean isApplet, loggedIn;
	private static final int LOGIN = 1, LOGOUT = 2, CLEAR = 3, OK = 4, CANCEL = 5;
	private JButton	jb_logout, jb_login , jb_clear;				
	private JButton jb_ok, jb_cancel;
	private JTextArea jta_chatDisplay;
	private JTextField jtf_chat_string, jtf_avatar_name;;
	private JDialog jd_chat_name;
	private JScrollPane scrollPane;
	private Server server = null;
	
	public ChatClient () throws RemoteException {
		UnicastRemoteObject.exportObject(this);
	}
	public void init() {
		isApplet = true;
		initUI();
		addListeners();
	}
	public void destroy() {
		try {
			if(server != null) server.removeReceiver(this);
		} catch(RemoteException ex) {}
	}
	
	private void createComponents() {
       		jb_logout= new JButton("Log Out");
       		jb_login = new JButton("Log In");
       		jb_clear = new JButton("Clear Messages");
       		jb_ok = new JButton("OK");
       		jb_cancel = new JButton("Cancel");
       		jtf_avatar_name = new JTextField(10);
       		jtf_chat_string = new JTextField();
       		jd_chat_name= new JDialog();
       		jta_chatDisplay = new JTextArea();
       	}
	private void initUI() {
		createComponents();
		jb_logout.setActionCommand(LOGOUT +"");
		jb_login.setActionCommand(LOGIN+"");
		jb_clear.setActionCommand(CLEAR+"");
		jb_ok.setActionCommand(OK +"");
		jb_cancel.setActionCommand(CANCEL + "");
		jb_login.setToolTipText("login with new chat name");
		jb_logout.setToolTipText("logout chatRoom");
		jb_clear.setToolTipText("clear chat messages since started...");

		jd_chat_name.setTitle("Input Avatar Name");
		JPanel dummy = new JPanel ();
		dummy.add(jb_ok);
		dummy.add(jb_cancel);
		JPanel p = new JPanel ();
	
		p.add(new Label("Avatar Name: "));
		p.add(jtf_avatar_name);
		jd_chat_name.getContentPane().add(p, BorderLayout.NORTH);
		jd_chat_name.getContentPane().add(dummy, BorderLayout.SOUTH);
		jd_chat_name.pack();
		
		jta_chatDisplay.setBackground(Color.black);
		jta_chatDisplay.setForeground(Color.green);
		jta_chatDisplay.append("Welcome ! RMI Callback Based ChatClient....\n");
		jta_chatDisplay.append("Click 'Log In' to enter chat,'Log Out' to Exit Chat Room.\n");
		jta_chatDisplay.setEditable(false);
		jb_logout.setEnabled(false);
		
		JPanel tmpPanel = new JPanel();
		tmpPanel.add(jb_login);
		tmpPanel.add(jb_logout);
		tmpPanel.add(jb_clear);
		scrollPane = new JScrollPane(jta_chatDisplay);
		
        	Container contentPane = getContentPane();
        	contentPane.setLayout(new BorderLayout());		
		contentPane.add(tmpPanel , BorderLayout.NORTH);
		contentPane.add(scrollPane, BorderLayout.CENTER);
		contentPane.add(jtf_chat_string, BorderLayout.SOUTH);
	}
	private void addListeners() {
		ActionListener tf_listener = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(!loggedIn) {
					jtf_chat_string.setText("Please Login First...");
					return;
				} 
				try {
					send(jtf_chat_string.getText());
					jtf_chat_string.setText("");
				} catch(RemoteException ex) {
					ex.printStackTrace();
				}
			}
		};
		jb_logout.addActionListener(this);
		jb_login.addActionListener(this);
		jb_clear.addActionListener(this);
		jb_ok.addActionListener(this);
		jb_cancel.addActionListener(this);
		jtf_chat_string.addActionListener(tf_listener);
	}
	private void openConnection() throws RemoteException {
		host = (!isApplet ? host: getDocumentBase().getHost());
		if(server == null) {
			try {		
				String rmiString = RmiUtil.getRmiString(Server.CHAT_SERVER, host, 2001);
				server = (Server)Naming.lookup(rmiString);
				System.out.println("Looked Up Remote Object...");
				server.addReceiver(this);
				send(login_name + " entered ...\n");
				loggedIn = true;
			} catch( Exception ex) {
				throw new RemoteException("Couldn't get a valid remote refrence ...["+ ex + "]");
			}
		}
		jb_login.setEnabled(false);
		jb_logout.setEnabled(true);
		jtf_chat_string.requestFocus();
		jtf_chat_string.setText("");
	}
	private void closeConnection() throws RemoteException {
		server.broadcast(" " + login_name + " has left the room.");  	
		server.removeReceiver(this);
		loggedIn = false;
		server = null;
		jb_logout.setEnabled(false);
		jb_login.setEnabled(true);
	}
	private void go(String host) {
		this.host = host;
		JFrame jf = new JFrame ();
		jf.getContentPane().add(this);
		init();
		isApplet = false;
		jf.setSize(350, 400);
		jf.setVisible(true);
		jf.addWindowListener(new WindowCloser());
	}
	private void send(String msg) throws RemoteException {
		server.broadcast("[" + login_name + "] " + msg);
		jtf_chat_string.setText("");
	}
	
       	public void printMsg(String msg) throws RemoteException {
       		jta_chatDisplay.append(msg+ "\n");
       		JScrollBar vScroll = null;                   
       		vScroll = scrollPane.getVerticalScrollBar();
       		vScroll.setValue(vScroll.getMaximum());     	
       	}

	public static void main(String [] args) {
		if(args.length < 1) {
			System.out.println("Usage: java pejb.rmi.chat.client.ChatClient hostName[ip]");
			System.exit(1);
		}
		try {
			new ChatClient().go(args[0]);
		} catch(RemoteException ex) {
			ex.printStackTrace();
		}
	}
	
//Inner Class Definition
	class WindowCloser extends WindowAdapter {
		public void windowClosing(WindowEvent e) {
			if(server != null) {
        			try {
					closeConnection();
        			} catch(RemoteException ex) {
        				ex.printStackTrace();
	        		}                                             
        		}                                                              
        		System.exit(0);
        	}
	}
// ActionListener event method...
	public void actionPerformed(ActionEvent e) {                                             
		int buttonId = Integer.parseInt(e.getActionCommand());
		switch(buttonId) {                                                                               
			case LOGOUT:
				try {                                                                              
					closeConnection();                                                         
				} catch (RemoteException ex) {                                                       
					ex.printStackTrace();                                                      
				}                                                                                 
			break;                                                                                     
			case LOGIN:                                                                                
				jd_chat_name.setLocation(ChatClient.this.getLocationOnScreen().x + 50,             
			 	ChatClient.this.getLocationOnScreen().y + 50);                                     
				jd_chat_name.setVisible(true);                                                     
			break;                                                                                     
			case OK:                                                                                   
				login_name = jtf_avatar_name.getText();
				try {                                                          
					openConnection();
					jta_chatDisplay.setText("Your Avatar Name Is " + login_name + ".\n");
					jd_chat_name.setVisible(false);
				} catch(RemoteException ex) {                                                      
					ex.printStackTrace();
				}   
			break;
			case CANCEL:
				jd_chat_name.setVisible(false);
			break;
			case CLEAR:
				jta_chatDisplay.setText("Cleard the diplay area.\n");
			break;
		}
	}
}