package pejb.ejb.session.board;
import pejb.ejb.entity.board.*;
import pejb.beans.board.BoardRec;
import pejb.ejb.base.*;
import javax.ejb.*;
import java.util.*;
import java.sql.*;
import java.rmi.*;

public class SearchRecBean extends SessionAdapter{
	private Connection conn;
	private ResultSet rset;
	private Statement stmt;
	private EJBUtil util;
	private BoardHome board_home;
	String jndi_enc ="";
	private int linesPerPage;
	
	public void ejbCreate() {
		print("ejbCreate Called");
	}

	/// Business Methods from remote interface ////////
	public int getTotalRows(String searchText, String searchCondition) throws RemoteException {
		int totalRows = -1;
		StringBuffer sb = new StringBuffer(125);
		sb.append("select count(\"recno\") from \"BoardBeanTable\"");
		sb.append(makeSearchString(searchText, searchCondition));
		try {
			conn = util.getConnection(jndi_enc);
			stmt = conn.createStatement();
			rset = stmt.executeQuery(sb.toString());
			if(rset.next()) {
				totalRows = rset.getInt(1);
			}
		} catch(SQLException e) {
			e.printStackTrace();
			throw new RemoteException(e.getMessage());
		} finally {
			close(rset, stmt, conn);
		}
		return totalRows;
	}
	
	public Vector getAllBoard(String searchText, String searchCondition, int currentPage) throws RemoteException {
		Vector boardRecs = new Vector(15);
		int totalRows = getTotalRows(searchText,searchCondition);
		int searchrow = getSearchRowCount(currentPage, totalRows);
		StringBuffer sb = new StringBuffer(256);
		sb.append("select \"recno\" from (select \"recno\", \"ref\", \"step\" from \"BoardBeanTable\"");
		sb.append(makeSearchString(searchText, searchCondition));
		sb.append(" order by \"ref\" desc, \"step\") where rownum <= ");
		sb.append( (currentPage * linesPerPage) + " order by \"ref\", \"step\" desc");
		try {
			conn = util.getConnection(jndi_enc);
			stmt = conn.createStatement();
			rset = stmt.executeQuery(sb.toString());
			for(int i = 0; rset.next() && i < searchrow ; i++) {
				int recno = rset.getInt(1);
				Board ejb = board_home.findByPrimaryKey(new BoardPK(recno));
				BoardRec record = new BoardRec();
				mapBoardRec(record, ejb);
				boardRecs.addElement(record);
			} 
		} catch(Exception e) {
			throw new RemoteException(e.getMessage());
		} finally {
			close(rset, stmt, conn);
		}  		
		return boardRecs;		
	}
	
	public BoardRec getBoard(int recno) throws RemoteException{
		Board ejb = null;
		try {
			ejb = board_home.findByPrimaryKey(new BoardPK(recno));
		} catch(FinderException e) {
			print("Such record with " + recno + " does not exists");
			throw new RemoteException(e.getMessage());
		}
		BoardRec record = new BoardRec();
		mapBoardRec(record, ejb);
		return record;
	}
	
	public Vector getRefBoard(int ref) throws RemoteException {
		Vector boardRecs = new Vector(4, 4);
		String query ="select  \"recno\" from \"BoardBeanTable\" where \"ref\" = " + ref + " order by \"ref\", \"step\" desc";
		try {
			conn = util.getConnection(jndi_enc);
			stmt = conn.createStatement();			
			rset = stmt.executeQuery(query);
			while(rset.next()) {
				int recno = rset.getInt(1);
				BoardRec record = new BoardRec();
				Board ejb = board_home.findByPrimaryKey(new BoardPK(recno));
				mapBoardRec(record, ejb);
				boardRecs.addElement(record);
			}
		} catch(Exception e) {
			throw new RemoteException(e.getMessage());
		} finally {
			close(rset, stmt, conn);
		}
		return boardRecs;
	}
	public void increaseHitCnt(int recno) throws RemoteException {
		String query = "update \"BoardBeanTable\" set \"hitCnt\" = \"hitCnt\" + 1 where \"recno\" = " + recno;
		try {
			conn = util.getConnection(jndi_enc);
			stmt = conn.createStatement();			
			stmt.executeUpdate(query);
		} catch(Exception e) {
			throw new RemoteException(e.getMessage());
		} finally {
			close(null, stmt, conn);
		}		
	}
	
	// overridden from SessionAdapter class
	public void setSessionContext(SessionContext ctx) throws RemoteException, EJBException{
		util = new EJBUtil();
		board_home = (BoardHome)util.lookup("java:comp/env/ejb/Board", BoardHome.class, EJBUtil.J2EESDK);
		this.jndi_enc = (String)util.getEnvEntry("ConnectionPool");
		Integer it_lines = (Integer)util.getEnvEntry("LinesPerPage");
		this.ejb_name = getClass().getName();
		this.linesPerPage = it_lines.intValue();
		this.debug = true;
		super.setSessionContext(ctx);
	}
	public void ejbRemove() {
		close(null, null, conn);
	}

	///////// Utilities //////////
	private String makeSearchString(String searchText, String searchCondition) {
		if(searchText == null || searchText.equals("")) return "";
		StringBuffer sb = new StringBuffer(125);
		if(!searchCondition.equals("contents")) {
		    sb.append(" where upper(\"" + searchCondition + "\") like upper('%" + searchText+ "%' ) ");
		} else {
		    sb.append(" where contains(\"contents\", '" + searchText + "') > 0 ");
		}
		return sb.toString();
	}
 	private int getSearchRowCount(int currentPage, int totalRows) {
		int searchRow = linesPerPage;
		int totalPage = (int)Math.ceil((double)totalRows / linesPerPage);
		if(currentPage == totalPage && (totalRows % linesPerPage != 0)) {
			searchRow = (totalRows % linesPerPage);
		}
		return searchRow;
	}
	private void mapBoardRec(BoardRec record, Board ejb) throws RemoteException {
		record.setRecno(((BoardPK)ejb.getPrimaryKey()).recno);
		record.setRef(ejb.getRef());
		record.setStep(ejb.getStep());
		record.setDepth(ejb.getDepth());
		record.setUserId(ejb.getUserId());
		record.setUserName(ejb.getUserName());
		record.setEmail(ejb.getEmail());
		record.setReplyCnt(ejb.getReplyCnt());
		record.setHitCnt(ejb.getHitCnt());
		record.setSubject(ejb.getSubject());
		record.setContents(ejb.getContents());
		record.setPasswd(ejb.getPasswd());
		record.setRegisteredDate(ejb.getRegisteredDate());	
	}
	private void close(ResultSet rset, Statement stmt, Connection conn) {
		try {
			if(rset != null) rset.close();
			if(stmt != null) stmt.close();
			if(conn != null) {
				conn.close();
				conn = null;
			}
		} catch(Exception e) { e.printStackTrace(); }
	}	
}